package de.htwsaar.webshop.service;

import de.htwsaar.webshop.repository.entities.Account;

import java.util.List;

public interface AccountService {
    List<Account> findAll();

    Account saveNew(Account account);

    Account save(Account account);

    boolean deleteIfExists(Account account);

    Account isValidLogin(String email, String password);

    boolean existsWithEmail(String email);
}